@@ -35,7 +35,9 @@
 #include "token.h"
 #include "fontfcn.h"
 #include "blues.h"
- 
+
+
+/* #define DEBUG_SCANFONT */
  
  
 static int rc;
@@ -50,7 +52,8 @@ static psobj *inputP;
 extern int T1Getc(F_FILE * );
 extern int T1Ungetc( int,F_FILE *);
 extern int SearchDictName( psdict *dictP, psobj *keyP);
-  
+extern void T1io_setblocklen( long len);
+
  
 /* functions from util.c */
 extern void objFormatInteger(psobj *objP, int value);
@@ -66,7 +69,7 @@ extern void objFormatFile( psobj *objP, FILE *valueP);
 /**********************************************************************/
 /*   Init_BuiltInEncoding()                                           */
 /*                                                                    */
-/*     Initializes the StandardEncoding and ISOLatin1Encoding vector. */
+/*     Initializes the StandardEncoding.                              */
 /*                                                                    */
 /**********************************************************************/
 typedef struct				/* Builtin Standard Encoding */
@@ -76,7 +79,7 @@ typedef struct				/* Builtin Standard Encoding */
 } EncodingTable;
 
 
-static char not_def[]=".notdef";
+char not_def[]=".notdef";
 
 
 static EncodingTable StdEnc[] = {
@@ -230,208 +233,13 @@ static EncodingTable StdEnc[] = {
    {0372 , "oe"},
    {0373 , "germandbls"},
    { 0,  0 }
-    
-};
-
-static EncodingTable ISO8859Enc[] = {
-   {32, "space"},
-   {33, "exclam"},
-   {34, "quotedbl"},
-   {35, "numbersign"},
-   {36, "dollar"},
-   {37, "percent"},
-   {38, "ampersand"},
-   {39, "quoteright"},
-   {40, "parenleft"},
-   {41, "parenright"},
-   {42, "asterisk"},
-   {43, "plus"},
-   {44, "comma"},
-   {45, "minus"},
-   {46, "period"},
-   {47, "slash"},
-   {48, "zero"},
-   {49, "one"},
-   {50, "two"},
-   {51, "three"},
-   {52, "four"},
-   {53, "five"},
-   {54, "six"},
-   {55, "seven"},
-   {56, "eight"},
-   {57, "nine"},
-   {58, "colon"},
-   {59, "semicolon"},
-   {60, "less"},
-   {61, "equal"},
-   {62, "greater"},
-   {63, "question"},
-   {64, "at"},
-   {65, "A"},
-   {66, "B"},
-   {67, "C"},
-   {68, "D"},
-   {69, "E"},
-   {70, "F"},
-   {71, "G"},
-   {72, "H"},
-   {73, "I"},
-   {74, "J"},
-   {75, "K"},
-   {76, "L"},
-   {77, "M"},
-   {78, "N"},
-   {79, "O"},
-   {80, "P"},
-   {81, "Q"},
-   {82, "R"},
-   {83, "S"},
-   {84, "T"},
-   {85, "U"},
-   {86, "V"},
-   {87, "W"},
-   {88, "X"},
-   {89, "Y"},
-   {90, "Z"},
-   {91, "bracketleft"},
-   {92, "backslash"},
-   {93, "bracketright"},
-   {94, "asciicircum"},
-   {95, "underscore"},
-   {96, "quoteleft"},
-   {97, "a"},
-   {98, "b"},
-   {99, "c"},
-   {100, "d"},
-   {101, "e"},
-   {102, "f"},
-   {103, "g"},
-   {104, "h"},
-   {105, "i"},
-   {106, "j"},
-   {107, "k"},
-   {108, "l"},
-   {109, "m"},
-   {110, "n"},
-   {111, "o"},
-   {112, "p"},
-   {113, "q"},
-   {114, "r"},
-   {115, "s"},
-   {116, "t"},
-   {117, "u"},
-   {118, "v"},
-   {119, "w"},
-   {120, "x"},
-   {121, "y"},
-   {122, "z"},
-   {123, "braceleft"},
-   {124, "bar"},
-   {125, "braceright"},
-   {126, "asciitilde"},
-   {161, "exclamdown"},
-   {162, "cent"},
-   {163, "sterling"},
-   {164, "currency"},
-   {165, "yen"},
-   {166, "brokenbar"},
-   {167, "section"},
-   {168, "dieresis"},
-   {169, "copyright"},
-   {170, "ordfeminine"},
-   {171, "guillemotleft"},
-   {172, "logicalnot"},
-   {173, "hyphen"},
-   {174, "registered"},
-   {175, "macron"},
-   {176, "degree"},
-   {177, "plusminus"},
-   {178, "twosuperior"},
-   {179, "threesuperior"},
-   {180, "acute"},
-   {181, "mu"},
-   {182, "paragraph"},
-   {183, "periodcentered"},
-   {184, "cedilla"},
-   {185, "onesuperior"},
-   {186, "ordmasculine"},
-   {187, "guillemotright"},
-   {188, "onequarter"},
-   {189, "onehalf"},
-   {190, "threequarters"},
-   {191, "questiondown"},
-   {192, "Agrave"},
-   {193, "Aacute"},
-   {194, "Acircumflex"},
-   {195, "Atilde"},
-   {196, "Adieresis"},
-   {197, "Aring"},
-   {198, "AE"},
-   {199, "Ccedilla"},
-   {200, "Egrave"},
-   {201, "Eacute"},
-   {202, "Ecircumflex"},
-   {203, "Edieresis"},
-   {204, "Igrave"},
-   {205, "Iacute"},
-   {206, "Icircumflex"},
-   {207, "Idieresis"},
-   {208, "Eth"},
-   {209, "Ntilde"},
-   {210, "Ograve"},
-   {211, "Oacute"},
-   {212, "Ocircumflex"},
-   {213, "Otilde"},
-   {214, "Odieresis"},
-   {215, "multiply"},
-   {216, "Oslash"},
-   {217, "Ugrave"},
-   {218, "Uacute"},
-   {219, "Ucircumflex"},
-   {220, "Udieresis"},
-   {221, "Yacute"},
-   {222, "Thorn"},
-   {223, "germandbls"},
-   {224, "agrave"},
-   {225, "aacute"},
-   {226, "acircumflex"},
-   {227, "atilde"},
-   {228, "adieresis"},
-   {229, "aring"},
-   {230, "ae"},
-   {231, "ccedilla"},
-   {232, "egrave"},
-   {233, "eacute"},
-   {234, "ecircumflex"},
-   {235, "edieresis"},
-   {236, "igrave"},
-   {237, "iacute"},
-   {238, "icircumflex"},
-   {239, "idieresis"},
-   {240, "eth"},
-   {241, "ntilde"},
-   {242, "ograve"},
-   {243, "oacute"},
-   {244, "ocircumflex"},
-   {245, "otilde"},
-   {246, "odieresis"},
-   {247, "divide"},
-   {248, "oslash"},
-   {249, "ugrave"},
-   {250, "uacute"},
-   {251, "ucircumflex"},
-   {252, "udieresis"},
-   {253, "yacute"},
-   {254, "thorn"},
-   {255, "ydieresis"},
-   { 0,  0 }
 };
 
 
 /* Encodings changed from static to global since they must be accessible
    by T1_LoadFont() when reallocating memory. */
 psobj *StdEncArrayP = NULL;
-psobj *ISOLatin1EncArrayP = NULL; 
+
 
 static psobj *MakeEncodingArrayP(encodingTable)
     EncodingTable *encodingTable;
@@ -459,9 +267,11 @@ static psobj *MakeEncodingArrayP(encodingTable)
  
 boolean Init_BuiltInEncoding()
 {
-    StdEncArrayP = MakeEncodingArrayP(StdEnc);
-    ISOLatin1EncArrayP = MakeEncodingArrayP(ISO8859Enc);
-    return (StdEncArrayP && ISOLatin1EncArrayP);
+  StdEncArrayP = MakeEncodingArrayP(StdEnc);
+  if (StdEncArrayP==NULL)
+    return( FALSE);
+  else
+    return( TRUE);
 }
  
 /********************************************************************/
@@ -501,17 +311,17 @@ static int getEncoding(arrayP)
 {
 
   scan_token(inputP);
-  if ((tokenType == TOKEN_NAME)
-                &&
-     (((tokenLength==16) && (!strncmp(tokenStartP,"StandardEncoding",16))) ||
-      (((tokenLength==17) && (!strncmp(tokenStartP,"ISOLatin1Encoding",17))))))
-  {
+  if ((tokenType == TOKEN_NAME) &&
+      ((tokenLength==16) && (!strncmp(tokenStartP,"StandardEncoding",16))) 
+      )
+    {
       /* Adobe Standard Encoding */
 
       if (tokenLength == 16)
 	  arrayP->data.valueP = (char *) StdEncArrayP;
       else
-	  arrayP->data.valueP = (char *) ISOLatin1EncArrayP;
+	return(SCAN_ERROR);
+      
 
       arrayP->len = 256;
       return(SCAN_OK);
@@ -560,7 +370,7 @@ static int getEncoding(arrayP)
       arrayP->len = 256;
 
       for (i=0; i<256; i++)
-	  objFormatName(objP + i, 7, ".notdef");
+	  objFormatName(objP + i, 7, not_def);
 
       while (TRUE)
       {
@@ -652,20 +462,24 @@ static int getArray(arrayP)
   /* NOTREACHED*/
 }
 /***================================================================***/
-static int getName(nameP)
-    char *nameP;
-{
-  do {
-    scan_token(inputP);
-    if (tokenType <= TOKEN_NONE) {
-      if (tokenTooLong) return(SCAN_OUT_OF_MEMORY);
-      return(SCAN_ERROR);
-    }
+/* is not needed */
+/*
+  static int getName(nameP)
+  char *nameP;
+  {
+  do { 
+  scan_token(inputP);
+  if (tokenType <= TOKEN_NONE) {
+  if (tokenTooLong) return(SCAN_OUT_OF_MEMORY);
+  return(SCAN_ERROR);
+  }
   } while ((tokenType != TOKEN_NAME) ||
-    (0 != strncmp(tokenStartP,nameP,strlen(nameP))) );
-  /* found */
+  (0 != strncmp(tokenStartP,nameP,strlen(nameP))) );
+  
   return(SCAN_OK);
-}
+  }
+*/
+
 /***================================================================***/
 static int getNbytes(N)
     int N;
@@ -713,7 +527,7 @@ static int getLiteralName(nameObjP)
  
 /***================================================================***/
 /*
- *   BuildSubrs routine
+ *   BuildSubrs routine 
  */
 /***================================================================***/
  
@@ -738,8 +552,19 @@ static int BuildSubrs(FontP)
      /* process all the Subrs, but do not update anything */
      /* can not just skip them because of the binary data */
      for (i=0;i<N;i++) {
-       /* look for dup */
-       rc = getName("dup");
+       /* look for "dup" or "ND" or "|-" or (noaccess) "def". If one of
+	  the latter three token appears, we are done even if still i < N-1.
+	  in this case, there arent as much subroutines as fields in the
+	  array which is allowed (Fix thanks to Derek B. Noonburg) */
+       do {
+	 rc = getNextValue(TOKEN_NAME);
+	 if ( rc != SCAN_OK ) return(rc);
+	 if (!strncmp(tokenStartP, "ND", 2) ||
+	     !strncmp(tokenStartP, "|-", 2) ||
+	     !strncmp(tokenStartP, "def", 3)) {
+	   return SCAN_OK;
+	 }
+       } while (strncmp(tokenStartP, "dup", 3));
        if (rc) return(rc);
        /* get 2 integers */
        I = getInt();
@@ -758,17 +583,30 @@ static int BuildSubrs(FontP)
  
    arrayP = (psobj *)vm_alloc(N*sizeof(psobj));
    if (!(arrayP) ) return(SCAN_OUT_OF_MEMORY);
+   /* explicitly resetting the array is not necessary since the pointer
+      as well as the subroutines both are located in VM and we can
+      assume that only define subroutines are referenced. */
    FontP->Subrs.len = N;
    FontP->Subrs.data.arrayP =  arrayP;
    /* get N values for Subrs */
    for (i=0;i<N;i++) {
-     /* look for dup */
-     rc = getName("dup");
-     if (rc) return(rc);
+     /* look for "dup" or "ND" or "|-" or (noaccess) "def". If one of
+	the latter three token appears, we are done even if still i < N-1.
+	in this case, there arent as much subroutines as fields in the
+	array which is allowed (Fix thanks to Derek B. Noonburg) */
+     do {
+       rc = getNextValue(TOKEN_NAME);
+       if ( rc != SCAN_OK ) return(rc);
+       if (!strncmp(tokenStartP, "ND", 2) ||
+	   !strncmp(tokenStartP, "|-", 2) ||
+	   !strncmp(tokenStartP, "def", 3)) {
+	 return SCAN_OK;
+       }
+     } while (strncmp(tokenStartP, "dup", 3));
      /* get 2 integers */
-     I = getInt();
-     if (rc) return(rc);
-     J = getInt();
+     I = getInt();       /* index into array of Subroutines */
+     if (rc) return(rc); 
+     J = getInt();       /* number of binary bytes that follow */
      if (rc) return(rc);
      if ( (I < 0) || (J < 0 ) ) return (SCAN_ERROR);
      arrayP[I].len = J;
@@ -785,6 +623,8 @@ static int BuildSubrs(FontP)
    return(SCAN_OK);
  
 }
+
+
 /***================================================================***/
 /***================================================================***/
 /*
@@ -827,6 +667,8 @@ static int BuildCharStrings(FontP)
    for (i=1;i<=N;i++) {
      /* look for next literal name  */
      rc = getLiteralName(&(dictP[i].key));
+     /* If rc=SCAN_END, the end of file has been reached. This means
+	there were less than N charstrings. This is perfectly valid. */
      if (rc) return(rc);
      /* get 1 integer */
      J = getInt();
@@ -841,7 +683,10 @@ static int BuildCharStrings(FontP)
        dictP[i].value.data.valueP = tokenStartP;
        if ( !(vm_alloc(J)) ) return(SCAN_OUT_OF_MEMORY);
      }
-     else return(rc);
+     else {
+       return(rc);
+     }
+     
    }
    return(SCAN_OK);
  
@@ -903,6 +748,8 @@ static int BuildFontInfo(fontP)
 /*
  *   BuildPrivate Dictionary
  */
+/* "LenIV" corrected to be "lenIV", otherwise fonts with some specific
+   lenIV value could not be decrypted. (RMz, Author of t1lib, 06/03/1998)*/
 /***================================================================***/
 static int BuildPrivate(fontP)
     psfont *fontP;
@@ -943,7 +790,7 @@ static int BuildPrivate(fontP)
   objFormatBoolean(&(Private[FORCEBOLD].value),DEFAULTFORCEBOLD);
   objFormatName(&(Private[LANGUAGEGROUP].key),13,"LanguageGroup");
   objFormatInteger(&(Private[LANGUAGEGROUP].value),DEFAULTLANGUAGEGROUP);
-  objFormatName(&(Private[LENIV].key),5,"LenIV");
+  objFormatName(&(Private[LENIV].key),5,"lenIV"); 
   objFormatInteger(&(Private[LENIV].value),DEFAULTLENIV);
   objFormatName(&(Private[RNDSTEMUP].key),9,"RndStemUp");
   objFormatBoolean(&(Private[RNDSTEMUP].value),DEFAULTRNDSTEMUP);
@@ -1208,10 +1055,13 @@ static int GetType1Blues(fontP)
  
   /* get the ptr to the lenIV entry */
   HintEntryP =  &(PrivateDictP[LENIV].value);
-  if (objPIsInteger(HintEntryP)) /* Must be integer! */
-      blues->lenIV = HintEntryP->data.integer;
-  else
-      blues->lenIV = DEFAULTLENIV;
+  if (objPIsInteger(HintEntryP)) {/* Must be integer! */
+    blues->lenIV = HintEntryP->data.integer;
+  }
+  else{
+    blues->lenIV = DEFAULTLENIV;
+  }
+  
  
   /* get the ptr to the ExpansionFactor entry */
   HintEntryP =  &(PrivateDictP[EXPANSIONFACTOR].value);
@@ -1368,18 +1218,10 @@ int scan_font(FontP)
  
  
   char   filename[128];
-  char   filetype[3];
   FILE   *fileP;
   char   *nameP;
   int    namelen;
-  int    V;
-  int    i;
-  boolean starthex80;
  
-    starthex80 = FALSE;
-    filetype[0] = 'r';
-    filetype[1] = 'b';
-    filetype[2] = '\0';
     /* copy the filename and remove leading or trailing blanks */
     /* point to name and search for leading blanks */
     nameP= FontP->FontFileName.data.nameP;
@@ -1399,15 +1241,7 @@ int scan_font(FontP)
     filterFile.data.fileP = NULL;
  
     inputP = &inputFile;
-    if ((fileP = fopen(filename,filetype))!=NULL) {
-      /* get the first byte of file */
-      V = getc(fileP);
-      /* if file starts with x'80' then skip next 5 bytes */
-      if ( V == 0X80 ) {
-        for (i=0;i<5;i++) V = getc(fileP);
-        starthex80 = TRUE;
-      }
-      else ungetc(V,fileP);
+    if ((fileP = fopen(filename,"rb"))!=NULL) {
       objFormatFile(inputP,fileP);
     }
     else {
@@ -1418,6 +1252,9 @@ int scan_font(FontP)
   InPrivateDict = FALSE;
   TwoSubrs      = FALSE;
   rc = BuildFontInfo(FontP);
+#ifdef DEBUG_SCANFONT
+  printf("BuildFontInfo(): retval=%d\n", rc);
+#endif
   if (rc != 0) return(rc);
  
   /* Assume everything will be OK */
@@ -1427,7 +1264,7 @@ int scan_font(FontP)
   do {
     /* Scan the next token */
     scan_token(inputP);
- 
+
     /* ==> tokenLength, tokenTooLong, tokenType, and tokenValue are */
     /* now set */
  
@@ -1442,17 +1279,33 @@ int scan_font(FontP)
       case TOKEN_LITERAL_NAME:
             /* Look up the name */
             tokenStartP[tokenLength] = '\0';
+	    /* At this point we check for the font not being a
+	       Multiple Master Font. If it is, we return an error.
+	       (RMz, 01/29/1999) */
+	    if (strncmp(tokenStartP, "BlendAxisTypes", 14)==0){
+	      rc=SCAN_MMFONT;
+	      break;
+	    }
             if (InPrivateDict ) {
               if (0== strncmp(tokenStartP,"Subrs",5) ) {
                 rc = BuildSubrs(FontP);
+#ifdef DEBUG_SCANFONT
+		printf("BuildSubrs(): retval=%d\n", rc);
+#endif
                 break;
               }
               if (0== strncmp(tokenStartP,"CharStrings",11) ) {
                 rc = BuildCharStrings(FontP);
+#ifdef DEBUG_SCANFONT
+		printf("BuildCharstrings(): retval=%d\n", rc);
+#endif
                 if ( (rc == SCAN_OK) ||(rc == SCAN_END) ) {
                   fclose(inputP->data.fileP);
                   /* Build the Blues Structure */
 		  rc = GetType1Blues(FontP);
+#ifdef DEBUG_SCANFONT
+		  printf("GetType1Blues(): retval=%d\n", rc);
+#endif
                   /* whatever the return code, return it */
                   /* all the work is done. This is the normal exit.*/
                   return(rc);
@@ -1486,22 +1339,19 @@ int scan_font(FontP)
         break;
       case TOKEN_NAME:
             if (0 == strncmp(tokenStartP,"eexec",5) ) {
-               /* if file started with x'80', check next 5 bytes */
-               if (starthex80) {
-                 V = getc(fileP);
-                 if ( V == 0X80 ) {
-                   for (i=0;i<5;i++) V = getc(fileP);
-                 }
-                 else ungetc(V,fileP);
-               }
-               filterFile.data.fileP = T1eexec(inputP->data.fileP);
-               if (filterFile.data.fileP == NULL) {
-                 fclose(inputFile.data.fileP);
-                 return(SCAN_FILE_OPEN_ERROR);
-               }
-               inputP = &filterFile;
- 
-               WantFontInfo = FALSE;
+	      /* We check for the pfb-headers in I/O-module */ 
+	      filterFile.data.fileP = T1eexec(inputP->data.fileP);
+	      if (filterFile.data.fileP == NULL) {
+		fclose(inputFile.data.fileP);
+		/* SCAN_FILE_OPEN_ERROR replaced because at this point
+		   a portion of the file has already been read successfully.
+		   We hence have encountered a premature end of file
+		   (2002-08-17, RMz). */
+		return SCAN_FILE_EOF;
+	      }
+	      inputP = &filterFile;
+	      
+	      WantFontInfo = FALSE;
             }
         break;
     }
